/*
  Copyright 2019 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.auth

import com.kakao.sdk.auth.model.AccessTokenResponse
import com.kakao.sdk.auth.model.OAuthToken
import com.kakao.sdk.auth.model.AgtResponse
import com.kakao.sdk.auth.network.rxKauth
import com.kakao.sdk.common.KakaoSdk
import com.kakao.sdk.common.SdkLogger
import com.kakao.sdk.common.network.ApiFactory
import com.kakao.sdk.common.rx

import io.reactivex.Single
import io.reactivex.SingleTransformer

/**
 * 카카오 OAuth 서버에서 제공하는 API 를 사용하기 위한 Reactive-X 클라이언트.
 */
class RxAuthApiClient(
    private val authApi: RxAuthApi = ApiFactory.rxKauth.create(RxAuthApi::class.java),
    private val accessTokenRepo: AccessTokenRepo = AccessTokenRepo.instance
) {

    /**
     * [RxAuthCodeClient] 를 이용하여 발급 받은 authorization code 를 사용하여 액세스 토큰을 발급한다.
     *
     * @param authCode authorization code
     *
     * @return [Single] instance that will emit [OAuthToken].
     */
    @JvmOverloads
    fun issueAccessToken(
        authCode: String,
        redirectUri: String = "kakao${KakaoSdk.applicationContextInfo.appKey}://oauth",
        clientId: String = KakaoSdk.applicationContextInfo.appKey,
        androidKeyHash: String = KakaoSdk.applicationContextInfo.signingKeyHash
    ): Single<OAuthToken> =
        authApi.issueAccessToken(
            clientId = clientId,
            redirectUri = redirectUri,
            androidKeyHash = androidKeyHash,
            authCode = authCode
        )
            .compose(handleAuthError())
            .map { OAuthToken.fromResponse(it) }
            .doOnSuccess { accessTokenRepo.setToken(it) }


    /**
     * 기존에 [issueAccessToken] 또는 이 메소드를 사용하여 발급 받은 리프레시 토큰으로 액세스 토큰을 갱신한다.
     *
     * @param refreshToken 리프레시 토큰
     *
     * @return [Single] instance that will emit [OAuthToken]
     */
    @JvmOverloads
    fun refreshAccessToken(
        refreshToken: String,
        clientId: String = KakaoSdk.applicationContextInfo.appKey,
        androidKeyHash: String = KakaoSdk.applicationContextInfo.signingKeyHash
    ): Single<OAuthToken> =
        authApi.issueAccessToken(
            clientId = clientId,
            redirectUri = null,
            androidKeyHash = androidKeyHash,
            refreshToken = refreshToken,
            grantType = Constants.REFRESH_TOKEN
        )
            .compose(handleAuthError())
            .map { OAuthToken.fromResponse(it) }
            .doOnSuccess { accessTokenRepo.setToken(it) }


    /**
     * @suppress
     */
    fun agt(clientId: String = KakaoSdk.applicationContextInfo.appKey): Single<AgtResponse> =
        Single.just(accessTokenRepo.getToken()?.accessToken)
            .flatMap { authApi.agt(clientId, it) }
            .compose(handleAuthError())

    @JvmSynthetic
    internal fun <T> handleAuthError(): SingleTransformer<T, T> = SingleTransformer {
        it.onErrorResumeNext { Single.error(AuthApiClient.translateError(it)) }
            .doOnError { SdkLogger.rx.e(it) }
            .doOnSuccess { SdkLogger.rx.i(it.toString()) }
    }
}

/**
 * Reactive-X 를 위한 [AuthApiClient] singleton 객체
 */
val AuthApiClient.Companion.rx by lazy { RxAuthApiClient() }