/*
  Copyright 2020 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.auth

import android.content.Context
import com.kakao.sdk.auth.model.OAuthToken
import java.util.concurrent.CountDownLatch

/**
 * 카카오 로그인을 실행하기 위한 클라이언트.
 * 카카오톡 또는 시스템 브라우저를 이용해 카카오계정 정보를 획득하고 API 요청에 필요한 사용자 토큰을 발급 받습니다.
 */
class LoginClient(
    private val authApiClient: AuthApiClient = AuthApiClient.instance,
    private val authCodeClient: AuthCodeClient = AuthCodeClient.instance
) {

    /**
     * 카카오톡 간편로그인 가능 여부(설치 여부)를 검사합니다.
     */
    fun isKakaoTalkLoginAvailable(context: Context): Boolean =
        authCodeClient.isKakaoTalkLoginAvailable(context)

    /**
     * 카카오톡 간편로그인으로 사용자를 인증하고 [OAuthToken] 을 발급 받습니다.
     *
     * @param context 간편로그인 Activity의 start를 수행하기 위한 현재 Activity context
     * @param callback 발급 받은 [OAuthToken] 을 반환합니다. 토큰은 현재 설정된 [AccessTokenRepo] 에 자동으로 저장됩니다.
     */
    @JvmOverloads
    fun loginWithKakaoTalk(
        context: Context,
        requestCode: Int = AuthCodeClient.DEFAULT_REQUEST_CODE,
        channelPublicIds: List<String>? = null,
        serviceTerms: List<String>? = null,
        autoLogin: Boolean? = null,
        callback: (token: OAuthToken?, error: Throwable?) -> Unit
    ) {
        authCodeClient.authorizeWithKakaoTalk(
            context,
            requestCode,
            channelPublicIds,
            serviceTerms,
            autoLogin
        ) { code, error ->
            code?.let {
                authApiClient.issueAccessToken(it) { token, error ->
                    callback(token, error)
                }
                return@authorizeWithKakaoTalk
            }
            callback(null, error)
        }
    }

    /**
     * 웹 브라우저(CustomTabs)에 있는 카카오계정 cookie 로 인증하여 새로운 [OAuthToken] 을 발급 받습니다.
     *
     * @param context Custom Tabs start를 수행하기 위한 현재 Activity context
     * @param callback 발급 받은 [OAuthToken] 을 반환합니다. 토큰은 현재 설정된 [AccessTokenRepo] 에 자동으로 저장됩니다.
     */
    @JvmOverloads
    fun loginWithKakaoAccount(
        context: Context,
        channelPublicIds: List<String>? = null,
        serviceTerms: List<String>? = null,
        autoLogin: Boolean? = null,
        callback: (token: OAuthToken?, error: Throwable?) -> Unit
    ) {
        authCodeClient.authorizeWithKakaoAccount(
            context,
            channelPublicIds,
            serviceTerms,
            autoLogin
        ) { code, error ->
            code?.let {
                authApiClient.issueAccessToken(it) { token, error ->
                    callback(token, error)
                }
                return@authorizeWithKakaoAccount
            }
            callback(null, error)
        }
    }

    /**
     * 현재 로그인된 사용자에게 아직 동의받지 못한 개인정보,접근권한 동의항목에 대하여 동의를 요청하고 동의항목이 업데이트 된 [OAuthToken] 을 발급 받습니다.
     *
     * @param context Custom Tabs start를 수행하기 위한 현재 Activity context
     * @param scopes 추가로 동의 받고자 하는 동의항목 ID 목록. 개발자사이트 동의항목 설정 화면에서 확인하실 수 있습니다.
     * @param callback 발급 받은 [OAuthToken] 을 반환합니다. 토큰은 현재 설정된 [AccessTokenRepo] 에 자동으로 저장됩니다.
     */
    fun loginWithNewScopes(
        context: Context,
        scopes: List<String>,
        callback: (token: OAuthToken?, error: Throwable?) -> Unit
    ) {
        authCodeClient.authorizeWithNewScopes(
            context,
            scopes
        ) { code, error ->
            code?.let {
                authApiClient.issueAccessToken(it) { token, error ->
                    callback(token, error)
                }
                return@authorizeWithNewScopes
            }
            callback(null, error)
        }
    }

    companion object {
        /**
         * 간편한 API 호출을 위해 기본 제공되는 singleton 객체
         */
        @JvmStatic
        val instance by lazy { LoginClient() }
    }
}