/*
  Copyright 2020 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.auth.network

import com.kakao.sdk.auth.AccessTokenRepo
import com.kakao.sdk.auth.LoginClient
import com.kakao.sdk.auth.network.AccessTokenInterceptor.Companion.requestWithAuthorization
import com.kakao.sdk.common.util.KakaoJson
import com.kakao.sdk.common.KakaoSdk
import com.kakao.sdk.common.model.ApiErrorResponse
import com.kakao.sdk.common.model.ApplicationContextInfo
import com.kakao.sdk.common.model.ClientError
import com.kakao.sdk.common.model.ClientErrorCause
import okhttp3.Interceptor
import okhttp3.Response
import java.util.concurrent.CountDownLatch

/**
 * @suppress
 *
 * 동적 동의
 */
class DynamicAgreementInterceptor(
    private val tokenRepo: AccessTokenRepo = AccessTokenRepo.instance,
    private val contextInfo: ApplicationContextInfo = KakaoSdk.applicationContextInfo
) : Interceptor {

    override fun intercept(chain: Interceptor.Chain): Response {
        val response = chain.proceed(chain.request())

        if (response.code() == 403) {
            val apiError = response.body()?.string()?.let {
                KakaoJson.fromJson<ApiErrorResponse>(it, ApiErrorResponse::class.java)
            }

            if (apiError?.requiredScopes != null) {
//                Log.e("#### DYNAMIC AGREEMENT", "${apiError.requiredScopes}")

                val latch = CountDownLatch(1)
                var newAgreementError: Throwable? = null
                LoginClient.instance.loginWithNewScopes(contextInfo.context, apiError.requiredScopes!!) { _, error ->  // TODO: 콜백의 토큰을 사용할지 repo의 토큰을 사용할지 고민
                    newAgreementError = error
                    latch.countDown()
                }
                latch.await()

                if (newAgreementError != null) {
                    throw java.io.IOException("Kakao SDK Error", newAgreementError!!)
                }

                return tokenRepo.getToken()?.accessToken?.let {
                    chain.proceed(
                        requestWithAuthorization(
                            response.request(),
                            it
                        )
                    )
                } ?: throw java.io.IOException("Kakao SDK Error", ClientError(ClientErrorCause.TokenNotFound, "not found dynamic agreement token"))
            }
        }

        return response
    }
}