/*
  Copyright 2020 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.auth.network

import com.kakao.sdk.auth.AccessTokenRepo
import com.kakao.sdk.auth.AuthApiClient
import com.kakao.sdk.auth.model.OAuthToken
import com.kakao.sdk.auth.network.AccessTokenInterceptor.Companion.requestWithAuthorization
import com.kakao.sdk.common.Constants
import okhttp3.Authenticator
import okhttp3.Request
import okhttp3.Response
import okhttp3.Route
import java.util.concurrent.CountDownLatch

/**
 * @suppress
 *
 * 401 에러 발생 시 진입하는 곳
 * 토큰을 갱신하고 갱신된 토큰을 추가한 request를 리턴하면 이를 기반으로 재시도 함
 */
class RefreshTokenAuthenticator(
    private val tokenRepo: AccessTokenRepo = AccessTokenRepo.instance,
    private val authApiClient: AuthApiClient = AuthApiClient.instance
) : Authenticator {
    override fun authenticate(route: Route?, response: Response): Request? {

        // 뒤이어 들어오는 401 응답들 pending
        synchronized(this) {
//            Log.e("#### AUTHENTICATOR", "$route")
//            Log.e("#### AUTHENTICATOR", "$response")

            val originalRequest = response.request()
            val originalAuthorizationHeader = originalRequest.header(Constants.AUTHORIZATION)

            if (originalAuthorizationHeader == null) {
//                Log.e("#### AUTHENTICATOR", "원본 요청에 토큰 없었음")
                return null
            }

            val accessToken = tokenRepo.getToken()?.accessToken
            if (accessToken == null) {
//                Log.e("#### AUTHENTICATOR", "현재 access_token 없음")
                return null
            }

            var newRequest = requestWithAuthorization(originalRequest, accessToken)
            val newAuthorizationHeader = newRequest.header(Constants.AUTHORIZATION)

//            Log.e("#### AUTHENTICATOR", "ORIGINAL: ${originalRequest.header(Constants.AUTHORIZATION)}")
//            Log.e("#### AUTHENTICATOR", "NEW:      ${newRequest.header(Constants.AUTHORIZATION)}")

            if (originalAuthorizationHeader != newAuthorizationHeader) {
//                Log.e("#### AUTHENTICATOR", "토큰이 변경(갱신)되었으므로 그대로 실행")   // pending resume
                return newRequest
            }

            val refreshToken = tokenRepo.getToken()?.refreshToken
            if (refreshToken == null) {
//                Log.e("#### AUTHENTICATOR", "refresh_token 없음")
                return null
            }

//            Log.e("#### AUTHENTICATOR", "토큰 갱신 시도...")
            val latch = CountDownLatch(1)
            var newToken: OAuthToken? = null
            authApiClient.refreshAccessToken(refreshToken) { token, error ->
                newToken = token
                latch.countDown()
            }
            latch.await()

            return newToken?.accessToken?.let {
//                Log.e("#### AUTHENTICATOR", "갱신 성공!!!  재시도 중... $it")

                // 신규 토큰 추가
                requestWithAuthorization(originalRequest, it)
            }


//            if (originalRequest.header(Constants.AUTHORIZATION) != null) {
//                Log.e("#### AUTHENTICATOR", "만료된 토큰이었음")
//
//                return tokenRepo.getToken().accessToken?.let {
//                    var newRequest: Request? = requestWithAuthorization(response.request(), it)
//
//                    Log.e("#### AUTHENTICATOR", "ORIGINAL: ${originalRequest.header(Constants.AUTHORIZATION)}")
//                    Log.e("#### AUTHENTICATOR", "CURRENT:  ${newRequest?.header(Constants.AUTHORIZATION)}")
//
//                    if (originalRequest.header(Constants.AUTHORIZATION).equals(newRequest?.header(Constants.AUTHORIZATION))) {
//
//                        newRequest = tokenRepo.getToken().refreshToken?.let {
//                            Log.e("#### AUTHENTICATOR", "토큰 갱신 시도...")
//
//                            val latch = CountDownLatch(1)
//                            authApiClient.refreshAccessToken(it) { response, error ->
//                                if (error != null) {
//                                    if (error is OAuthError && error.statusCode == 401) {
//                                        tokenRepo.clear()
//                                    }
//                                }
//                                latch.countDown()
//                            }
//                            latch.await()
//
//                            tokenRepo.getToken().accessToken?.let {
//                                Log.e("#### AUTHENTICATOR", "갱신 성공!!!  재시도 중... $it")
//
//                                // 신규 토큰 추가
//                                requestWithAuthorization(originalRequest, it)
//                            }
//                        }
//                    } else {
//                        Log.e("#### AUTHENTICATOR", "이미 갱신되어 있으므로 그대로 실행")
//                    }
//
//                    newRequest
//                }
//
//            } else {
//
//                return null
//            }
        }

    }
}
