/*
  Copyright 2019 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.user

import android.annotation.SuppressLint
import com.kakao.sdk.auth.AccessTokenRepo
import com.kakao.sdk.auth.LoginClient
import com.kakao.sdk.auth.network.kapiWithOAuth
import com.kakao.sdk.common.KakaoSdk
import com.kakao.sdk.common.model.KakaoSdkError
import com.kakao.sdk.common.network.ApiCallback
import com.kakao.sdk.common.network.ApiFactory
import com.kakao.sdk.user.model.AccessTokenInfo
import com.kakao.sdk.user.model.User
import com.kakao.sdk.user.model.UserServiceTerms
import com.kakao.sdk.user.model.UserShippingAddresses
import retrofit2.Call
import retrofit2.Callback
import retrofit2.HttpException
import retrofit2.Response
import java.util.*
import java.util.concurrent.CompletableFuture
import java.util.concurrent.Future

/**
 * 카카오 Open API의 사용자관리 API 호출을 담당하는 클라이언트.
 *
 * @since 2.0.0
 */
class UserApiClient (
    private val userApi: UserApi = ApiFactory.kapiWithOAuth.create(UserApi::class.java),
    private val accessTokenRepo: AccessTokenRepo = AccessTokenRepo.instance
) {

    /**
     * 사용자에 대한 다양한 정보를 얻을 수 있습니다.
     *
     * @since 2.0.0
     */
    @JvmOverloads
    fun me(secureReSource: Boolean = true, callback: (user: User?, error: Throwable?) -> Unit) {
        userApi.me(secureReSource)
            .enqueue(object: ApiCallback<User>() {
                override fun onComplete(model: User?, error: Throwable?) {
                    callback(model, error)
                }
            })
    }

    /**
     * 현재 토큰의 기본적인 정보를 조회합니다.
     *
     * [me] 에서 제공되는 다양한 사용자 정보 없이 가볍게 토큰의 유효성을 체크하는 용도로 사용하는 경우 추천합니다.
     * 액세스토큰이 만려되어있는 경우 리프레시토큰으로 갱신된 새로운 액세스토큰의 정보를 반환합니다.
     *
     * @since 2.0.0
     */
    fun accessTokenInfo(callback: (info: AccessTokenInfo?, error: Throwable?) -> Unit) {
        userApi.accessTokenInfo()
            .enqueue(object: ApiCallback<AccessTokenInfo>() {
                override fun onComplete(model: AccessTokenInfo?, error: Throwable?) {
                    callback(model, error)
                }
            })
    }

    /**
     * User 클래스에서 제공되고 있는 사용자의 부가정보를 신규저장 및 수정할 수 있습니다.
     *
     * 저장 가능한 키 이름은 개발자사이트 앱 설정의 사용자 관리 > 사용자 목록 및 프로퍼티 메뉴에서 확인하실 수 있습니다.
     * 앱 연결 시 기본 저장되는 nickname, profile_image, thumbnail_image 값도 덮어쓰기 가능하며 새로운 컬럼을 추가하면 해당 키 이름으로 값을 저장할 수 있습니다.
     *
     * @since 2.0.0
     */
    fun updateProfile(properties: Map<String, String>, callback: (error: Throwable?) -> Unit) {
        userApi.updateProfile(properties)
            .enqueue(object: ApiCallback<Unit>() {
                override fun onComplete(model: Unit?, error: Throwable?) {
                    callback(error)
                }
            })
    }

    /**
     * 토큰을 강제로 만료시킵니다. 같은 사용자가 여러개의 토큰을 발급 받은 경우 로그아웃 요청에 사용된 토큰만 만료됩니다.
     *
     * 클라이언트에 저장되어 있는 액세스토큰과 리프레시토큰을 삭제합니다.
     *
     * @since 2.0.0
     */
    fun logout(callback: (error: Throwable?) -> Unit) {
        userApi.logout()
            .enqueue(object: ApiCallback<Unit>() {
                override fun onComplete(model: Unit?, error: Throwable?) {
                    accessTokenRepo.clear()
                    callback(error)
                }
            })
    }

    /**
     * 카카오 플랫폼 서비스와 앱 연결을 해제합니다.
     *
     * 클라이언트에 저장되어 있는 액세스토큰과 리프레시토큰을 삭제합니다.
     *
     * @since 2.0.0
     */
    fun unlink(callback: (error: Throwable?) -> Unit) {
        userApi.unlink()
            .enqueue(object: ApiCallback<Unit>() {
                override fun onComplete(model: Unit?, error: Throwable?) {
                    if (error == null) {
                        accessTokenRepo.clear()
                    }
                    callback(error)
                }
            })
    }

    /**
     * 앱에 가입한 사용자의 배송지 정보를 얻어갑니다.
     *
     * @param fromUpdateAt
     * @param pageSize
     *
     * @since 2.0.0
     */
    @JvmOverloads
    fun shippingAddresses(
        fromUpdateAt: Date? = null,
        pageSize: Int? = null,
        callback: (userShippingAddresses: UserShippingAddresses?, error: Throwable?) -> Unit
    ) {
        userApi.shippingAddresses(fromUpdateAt = fromUpdateAt, pageSize = pageSize)
            .enqueue(object: ApiCallback<UserShippingAddresses>() {
                override fun onComplete(model: UserShippingAddresses?, error: Throwable?) {
                    callback(model, error)
                }
            })
    }

    /**
     * 앱에 가입한 사용자의 배송지 정보를 얻어갑니다.
     *
     * @param addressId 가져올 배송지 id
     *
     * @since 2.0.0
     */
    fun shippingAddresses(
        addressId: Long,
        callback: (userShippingAddresses: UserShippingAddresses?, error: Throwable?) -> Unit
    ) {
        userApi.shippingAddresses(addressId)
            .enqueue(object: ApiCallback<UserShippingAddresses>() {
                override fun onComplete(model: UserShippingAddresses?, error: Throwable?) {
                    callback(model, error)
                }
            })
    }

    /**
     * 사용자가 카카오 간편가입을 통해 동의한 서비스 약관 내역을 반환합니다.
     *
     * @since 2.0.0
     */
    fun serviceTerms(callback: (userServiceTerms: UserServiceTerms?, error: Throwable?) -> Unit) {
        userApi.serviceTerms()
            .enqueue(object: ApiCallback<UserServiceTerms>() {
                override fun onComplete(model: UserServiceTerms?, error: Throwable?) {
                    callback(model, error)
                }
            })
    }

    companion object {
        /**
         * 간편한 API 호출을 위해 기본 제공되는 singleton 객체
         */
        @JvmStatic
        val instance by lazy { UserApiClient() }
    }
}
